package hu.uw.pallergabor.irblegw;

import android.app.Activity;
import android.app.Service;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothGatt;
import android.bluetooth.BluetoothGattCallback;
import android.bluetooth.BluetoothGattCharacteristic;
import android.bluetooth.BluetoothGattDescriptor;
import android.bluetooth.BluetoothGattService;
import android.bluetooth.BluetoothManager;
import android.bluetooth.BluetoothProfile;
import android.content.Context;
import android.content.Intent;
import android.media.AudioManager;
import android.os.IBinder;
import android.util.Log;
import android.widget.Toast;

import java.util.UUID;

/**
 * Created by paller on 2015.12.01..
 */
public class IRGWService extends Service {
    private static final long IR_KEY_VOLUME_PLUS = 0x20df12edL;
    private static final long IR_KEY_VOLUME_MINUS = 0x20dfaa55L;
    private static final long IR_KEY_VOLUME_MUTE = 0x20df0af5L;

    // RFduino 2220, 2221, 2222, 2223
    public static final UUID RFDUINO_SERVICE_UUID = UUID.fromString("00002220-0000-1000-8000-00805F9B34FB");
    public static final UUID RECEIVE_CHARACTERISTIC_UUID = UUID.fromString("00002221-0000-1000-8000-00805F9B34FB");
    public static final UUID SEND_CHARACTERISTIC_UUID = UUID.fromString("00002222-0000-1000-8000-00805F9B34FB");
    public static final UUID DISCONNECT_CHARACTERISTIC_UUID = UUID.fromString("00002223-0000-1000-8000-00805F9B34FB");
    // 0x2902 org.bluetooth.descriptor.gatt.client_characteristic_configuration.xml
    public final static UUID CLIENT_CHARACTERISTIC_CONFIGURATION_UUID = UUID.fromString("00002902-0000-1000-8000-00805F9B34FB");
    private String LOG_TAG = "IRGWService";
    private BluetoothAdapter bluetoothAdapter;
    private BluetoothDevice gattExplorerDevice;
    private BluetoothGatt gatt;
    private BluetoothGattService gattService;
    private BluetoothGattCharacteristic receiveCharacteristic;
    private BluetoothGattCharacteristic sendCharacteristic;
    private BluetoothGattCharacteristic disconnectCharacteristic;
    private MyGattCallback myGattCallback = new MyGattCallback();
    private boolean connected;
    private AudioManager audioManager;

    public int onStartCommand(Intent intent, int flags, int startId) {
        super.onStartCommand(intent, flags, startId);
        Log.d(LOG_TAG, "onStartCommand");
        String deviceAddress = intent.getStringExtra("deviceAddress");
        if (deviceAddress == null) {
            Log.d(LOG_TAG, "device==null");
            stopSelf();
        }
        final BluetoothManager bluetoothManager =
                (BluetoothManager) getSystemService(Context.BLUETOOTH_SERVICE);
        bluetoothAdapter = bluetoothManager.getAdapter();
        gattExplorerDevice = bluetoothAdapter.getRemoteDevice(deviceAddress);
        connected = false;
        audioManager = (AudioManager) getSystemService(AUDIO_SERVICE);
        gattExplorerDevice.connectGatt(
                this,
                false,
                myGattCallback);
        Log.d(LOG_TAG, "onStartCommand ends");
        return START_NOT_STICKY;
    }

    public void onDestroy() {
        super.onDestroy();
        Log.d(LOG_TAG, "onDestroy");
    }

    @Override
    public IBinder onBind(Intent intent) {
        return serviceBinder;
    }

    private void disconnect() {
        disconnectCharacteristic.setValue("");
        disconnectCharacteristic.setWriteType(BluetoothGattCharacteristic.WRITE_TYPE_NO_RESPONSE);
        gatt.writeCharacteristic(disconnectCharacteristic);
        gatt.disconnect();
        connected = false;
    }

    private final IRGWServiceIF.Stub serviceBinder =
            new IRGWServiceIF.Stub() {
                public boolean isConnected() {
                    return connected;
                }

                public void disconnect() {
                    IRGWService.this.disconnect();
                }
            };

    private class MyGattCallback extends BluetoothGattCallback {
        public void onConnectionStateChange(BluetoothGatt gatt, int status, int newState) {
            if( newState == BluetoothProfile.STATE_DISCONNECTED) {
                Log.d( LOG_TAG, "STATE_DISCONNECTED");
                IRGWService.this.gatt = null;
                connected = false;
                Intent intent = new Intent();
                intent.setAction(IRGWServiceReceiver.GW_DISCONNECTED);
                sendBroadcast(intent);
//                Toast.makeText(IRGWService.this,
//                        IRGWService.this.getString( R.string.ge_disconnected),
//                        Toast.LENGTH_LONG).show();
                IRGWService.this.stopSelf();
            } else
            if( newState == BluetoothProfile.STATE_CONNECTED) {
                Log.d( LOG_TAG, "STATE_CONNECTED");
                IRGWService.this.gatt = gatt;
                connected = true;
                Intent intent = new Intent();
                intent.setAction(IRGWServiceReceiver.GW_CONNECTED);
                sendBroadcast(intent);
//                Toast.makeText(IRGWService.this,
//                        IRGWService.this.getString( R.string.ge_connected),
//                        Toast.LENGTH_SHORT).show();
                boolean success = gatt.discoverServices();
                Log.d( LOG_TAG, "discoverServices: "+success);
            }
        }

        // New services discovered
        public void onServicesDiscovered(BluetoothGatt gatt, int status) {
            if (status == BluetoothGatt.GATT_SUCCESS) {
                Log.d( LOG_TAG, "services discovered");
                gattService = gatt.getService(RFDUINO_SERVICE_UUID);
                receiveCharacteristic = gattService.getCharacteristic(RECEIVE_CHARACTERISTIC_UUID);
                sendCharacteristic = gattService.getCharacteristic(SEND_CHARACTERISTIC_UUID);
                disconnectCharacteristic = gattService.getCharacteristic(DISCONNECT_CHARACTERISTIC_UUID);
                gatt.setCharacteristicNotification(receiveCharacteristic, true);
                BluetoothGattDescriptor receiveConfigDescriptor = receiveCharacteristic.getDescriptor(CLIENT_CHARACTERISTIC_CONFIGURATION_UUID);
                if (receiveConfigDescriptor != null) {
                    receiveConfigDescriptor.setValue(BluetoothGattDescriptor.ENABLE_NOTIFICATION_VALUE);
                    gatt.writeDescriptor(receiveConfigDescriptor);
                } else {
                    Log.e(LOG_TAG, "Receive Characteristic can not be configured.");
                }
                Intent intent = new Intent();
                intent.setAction(IRGWServiceReceiver.GW_DISCOVERED);
                sendBroadcast(intent);

//                Toast.makeText(IRGWService.this,
//                        IRGWService.this.getString( R.string.ge_discovered),
//                        Toast.LENGTH_SHORT).show();
            } else {
                Log.w(LOG_TAG, "onServicesDiscovered received: " + status);
            }
        }

        public void onCharacteristicWrite (
                BluetoothGatt gatt,
                BluetoothGattCharacteristic characteristic,
                int status) {
            String success = status == BluetoothGatt.GATT_SUCCESS ?
                    "Success" :
                    "Failure";
            String message =
                    "Write operation on "+
                            characteristic.getUuid().toString()+
                            " : "+success;
            Log.d( LOG_TAG, "status: "+status+" : "+message);
        }

        public void onCharacteristicChanged (
                BluetoothGatt gatt,
                BluetoothGattCharacteristic characteristic) {
            Object o = characteristic.getValue();
            String message =
                    "Characteristic changed: "+
                            characteristic.getUuid().toString();
            if( o instanceof byte[] ) {
                byte b[] = (byte[])o;
                if( b.length == 4 ) {
                    int b0 = ((int)b[0]) & 0xFF;
                    int b1 = ((int)b[1]) & 0xFF;
                    int b2 = ((int) b[2]) & 0xFF;
                    int b3 = ((int)b[3]) & 0xFF;
                    long v = ((long)((b0) | (b1<<8) | (b2<<16) | (b3<<24))) & 0xFFFFFFFFL;
                    Log.d(LOG_TAG, "key received: " + Long.toHexString(v));
                    Intent intent = new Intent();
                    intent.setAction(IRGWServiceReceiver.GW_KEY);
                    intent.putExtra("key", v);
                    sendBroadcast(intent);
                    if( v == IR_KEY_VOLUME_PLUS ) {
                        Log.d(LOG_TAG, "IR_KEY_VOLUME_PLUS");
                        audioManager.adjustStreamVolume(
                                AudioManager.STREAM_MUSIC,
                                AudioManager.ADJUST_RAISE,
                                AudioManager.FLAG_SHOW_UI);
                    } else
                    if( v == IR_KEY_VOLUME_MINUS ) {
                        Log.d(LOG_TAG, "IR_KEY_VOLUME_MINUS");
                        audioManager.adjustStreamVolume(
                                AudioManager.STREAM_MUSIC,
                                AudioManager.ADJUST_LOWER,
                                AudioManager.FLAG_SHOW_UI);
                    } else
                    if( v == IR_KEY_VOLUME_MUTE ) {
                            Log.d( LOG_TAG, "IR_KEY_VOLUME_MUTE");
                            audioManager.adjustStreamVolume(
                                    AudioManager.STREAM_MUSIC,
                                    101,
                                    AudioManager.FLAG_SHOW_UI);
                    }
                }
            } else
                message = message + o.toString();
            Log.d( LOG_TAG, message);
        }
    }

}
